@@ -33,9 +33,11 @@ class PathError(Exception):
     def __str__(self):
         return self.desc % self.path
 
+
 class NonExistenceError(PathError):
     defaultdesc = "%s does not exist"
 
+
 class WrongTypeError(PathError):
     def __init__(self, path, type):
         self.path = path
@@ -44,6 +46,7 @@ class WrongTypeError(PathError):
     def __str__(self):
         return '%s is not a %s' % (self.path, self.type)
 
+
 class WouldNotOverwriteError(PathError):
     defaultdesc = "%s already exists - would not overwrite"
 
@@ -65,8 +68,13 @@ class _StatProperty(object):
 
 
 class PathObject(object):
-    def __init__(self, path):
-        self.path = _normalize(path)
+    @classmethod
+    def exists(cls, path):
+        return os.path.exists(path)
+    
+    def __init__(self, path, check=True):
+        if check: path = _normalize(path)
+        self.path = path
 
     def __str__(self):
         return self.path
@@ -75,7 +83,7 @@ class PathObject(object):
         return unicode(self.path)
 
     def __repr__(self):
-        return "%s '%s'" % (self.__class__.__name__, self.path)
+        return "%s(%r)" % (self.__class__.__name__, self.path)
 
     def __hash__(self):
         return hash(self.path)
@@ -91,21 +99,28 @@ class PathObject(object):
     def __ne__(self, other):
         if not isinstance(other, PathObject):
             return NotImplemented
-        return self.path <> other.path
+        return self.path != other.path
 
     @property
     def parentpath(self):
-        return p.split(self.loc)[0]
+        return p.split(self.path)[0]
 
     def parent(self):
         return Folder(p.split(self.path)[0])
 
+    @property
     def parts(self):
+        if self.path == '/':
+            return ('',)
         return tuple(self.path.split(p.sep))
 
+    @property
+    def fragment(self):
+        return PathFragment(self.parts)
+
     def isancestor(self, other):
-        myparts = self.parts()
-        itsparts = other.parts()
+        myparts = self.parts
+        itsparts = other.parts
         if len(myparts) > len(itsparts):
             return False
         else:
@@ -113,7 +128,7 @@ class PathObject(object):
                 return True
             else:
                 return False
-    
+
     def matches(self, pattern):
         return fnmatch.fnmatch(self.path, pattern)
     
@@ -121,8 +136,24 @@ class PathObject(object):
         return p.realpath(self.path)
     
     def linkdest(self):
-        if p.islink(self):
-            return os.readlink(self)
+        if p.islink(self.path):
+            return os.readlink(self.path)
+
+    def fragment_from(self, path):
+        other = new(path)
+        if not other.isancestor(self):
+            return None
+        return PathFragment(self.parts[len(other.parts):])
+
+    def __getitem__(self, item):
+        if isinstance(item, slice):
+            return PathFragment(self.parts[item])
+        elif isinstance(item, int):
+            return self.parts[item]
+        raise TypeError("Must use ints to get components of paths")
+
+    def __getslice__(self, i, j):
+        return PathFragment(self.parts[i:j])
 
     def __gt__(self, other):
         if not isinstance(other, PathObject):
@@ -151,30 +182,35 @@ class PathObject(object):
 
 
 class Folder(PathObject):
-    def __init__(self, path=None, mkdir=False):
-        if path is None:
-            path = os.getcwd()
-        path = _normalize(path)
-        if not p.isdir(path):
-            if mkdir:
-                if not p.exists(path):
-                    os.makedirs(path)
-                else:
+    @classmethod
+    def exists(cls, path):
+        return os.path.isdir(path)
+    
+    def __init__(self, path=None, mkdir=False, check=True):
+        if check:
+            if path is None:
+                path = os.getcwd()
+            path = _normalize(path)
+            if not p.isdir(path):
+                if mkdir:
+                    if not p.exists(path):
+                        os.makedirs(path)
+                    else:
+                        raise WrongTypeError(path, 'folder')
+                elif p.exists(path):
                     raise WrongTypeError(path, 'folder')
-            elif p.exists(path):
-                raise WrongTypeError(path, 'folder')
-            else:
-                raise NonExistenceError(path)
+                else:
+                    raise NonExistenceError(path)
         self.path = path
 
-    def childpath(self, name):
-        return p.join(self.path, name)
+    def childpath(self, *parts):
+        return p.join(self.path, *parts)
 
-    def child(self, name):
-        if name == '..':
+    def child(self, *parts):
+        if parts == ('..',):
             return self.parent()
         else:
-            return new(p.join(self.path, name))
+            return new(p.join(self.path, *parts))
 
     def haschild(self, name):
         return p.exists(self.childpath(name))
@@ -199,7 +235,7 @@ class Folder(PathObject):
     
     def copy(self, dest, ignore):
         if isinstance(dest, basestring):
-            dest = Folder(basestring, mkdir=True)
+            dest = Folder(dest, mkdir=True)
         for item in self.ls():
             if item.isdir and not ignore(item):
                 dest.mkdir(item.basename)
@@ -218,6 +254,8 @@ class Folder(PathObject):
                                     "recursively")
             else:
                 os.rmdir(c.path)
+        else:
+            os.unlink(name)
         del c
         return
 
@@ -226,25 +264,33 @@ class Folder(PathObject):
 
     def ls(self, pattern=None, kind=ALL):
         if pattern:
-            return fnmatch.filter([self.child(n)
-                for n in os.listdir(self.path) if kind(n)], pattern)
+            return [self.child(n) for n in os.listdir(self.path)
+                    if kind(n) and fnmatch.fnmatch(n, pattern)]
         else:
             return [self.child(n) for n in os.listdir(self.path) if kind(n)]
 
     def names(self, pattern=None, kind=ALL):
         if pattern:
-            return fnmatch.filter([n for n in os.listdir(self.path)
-                if kind(n)], pattern)
+            return [n for n in os.listdir(self.path)
+                    if kind(n) and fnmatch.fnmatch(n, pattern)]
         else:
             return [n for n in os.listdir(self.path) if kind(n)]
 
     def paths(self, pattern=None, kind=ALL):
         if pattern:
-            return fnmatch.filter([self.childpath(n)
-                for n in os.listdir(self.path) if kind(n)], pattern)
+            return [self.childpath(n) for n in os.listdir(self.path)
+                    if kind(n) and fnmatch.fnmatch(n, pattern)]
         else:
             return [self.childpath(n) for n in os.listdir(self.path)
-                if kind(n)]
+                    if kind(n)]
+
+    def __add__(self, part):
+        if isinstance(part, PathFragment):
+            return self.childpath(*part.comps)
+        elif isinstance(part, tuple):
+            return self.childpath(*part)
+        else:
+            return self.childpath(part)
 
     def __mod__(self, pattern):
         return self.ls(pattern)
@@ -252,14 +298,22 @@ class Folder(PathObject):
     def __iter__(self):
         return iter(self.ls())
 
-    def __getitem__(self, name):
-        if isinstance(name, tuple):
-            return self.childpath(name[0])
+    def __getitem__(self, item):
+        if isinstance(item, PathFragment):
+            return self.child(*item.comps)
+        elif isinstance(item, slice):
+            return self.parts[item]
+        elif isinstance(item, int):
+            return self.parts[item]
+        elif isinstance(item, basestring):
+            return self.child(item)
+        elif isinstance(item, tuple):
+            return self.child(*item)
         else:
-            return self.child(name)
+            raise TypeError("Must use slice, int, string, or tuple")
     
     def __getslice__(self, i, j):
-        raise TypeError("Cannot slice a Folder object")
+        return PathFragment(self.parts[i:j])
 
     def __contains__(self, name):
         return self.haschild(name)
@@ -268,10 +322,15 @@ class Folder(PathObject):
 
 
 class File(PathObject):
-    def __init__(self, path):
-        path = _normalize(path)
-        if not p.isfile(path):
-            raise WrongTypeError(path, 'file')
+    @classmethod
+    def exists(cls, path):
+        return os.path.isfile(path)
+    
+    def __init__(self, path, check=False):
+        if check:
+            path = _normalize(path)
+            if not p.isfile(path):
+                raise WrongTypeError(path, 'file')
         self.path = path
         self._stats = None
 
@@ -299,7 +358,7 @@ class File(PathObject):
     
     def move(self, dest, overwrite=True):
         if isinstance(dest, Folder):
-            dest = dest[self.basename,]
+            dest = dest + self.basename
         elif p.isdir(dest):
             dest = p.join(dest, self.basename)
         if (not overwrite) and p.exists(dest):
@@ -367,19 +426,65 @@ class File(PathObject):
 
 class SomethingElse(PathObject):
     def __init__(self, path):
-        self.path = _normalize(path)
+        self.path = path
+
+
+class PathFragment(object):
+    """
+    This represents multiple path components (fragments of a path).
+    """
+    def __init__(self, comps):
+        if isinstance(comps, basestring):
+            self.comps = tuple(comps.split(p.sep))
+        else:
+            self.comps = tuple(comps)
+    
+    @classmethod
+    def from_urlpath(self, urlpath):
+        if urlpath.startswith('/'):
+            return PathFragment(urlpath[1:].split('/'))
+        else:
+            return PathFragment(urlpath.split('/'))
+    
+    def __getitem__(self, item):
+        if isinstance(item, (int, slice)):
+            return self.comps[item]
+    
+    def __getslice__(self, i, j):
+        return self.comps[i:j]
+    
+    def __iter__(self):
+        return iter(self.comps)
+    
+    def __add__(self, seq):
+        return self.__class__(self.comps + tuple(seq))
+    
+    def __radd__(self, e):
+        if isinstance(e, basestring):
+            return p.join(e, str(self))
+        return NotImplemented
+    
+    def __str__(self):
+        return p.sep.join(self.comps)
+    
+    def __repr__(self):
+        return '%s(%r)' % (self.__class__.__name__, self.comps)
+    
+    str = property(lambda self: p.sep.join(self.comps))
 
 
 def new(path):
     if isinstance(path, PathObject):
         return path
+    if isinstance(path, PathFragment):
+        path = str(path)
     path = _normalize(path)
     if p.isdir(path):
-        return Folder(path)
+        return Folder(path, check=False)
     elif p.isfile(path):
-        return File(path)
+        return File(path, check=False)
     elif p.exists(path):
-        return SomethingElse(path)
+        return SomethingElse(path, check=False)
     else:
         raise NonExistenceError(path)
 
